import Map from 'ol/Map';
import { OSM, XYZ, Vector as VectorSource } from 'ol/source';
import { Group as LayerGroup, Tile as TileLayer, Vector as VectorLayer } from 'ol/layer';
import { getCenter } from 'ol/extent';
import Feature from 'ol/Feature';
import Geolocation from 'ol/Geolocation';
import Point from 'ol/geom/Point';
import View from 'ol/View';
import LayerSwitcher from 'ol-layerswitcher';
import { Circle as CircleStyle, Fill, Stroke, Style } from 'ol/style';
import { defaults as defaultControls, Control, ScaleLine, ZoomToExtent } from 'ol/control';
import { defaults as defaultInteractions } from 'ol/interaction';
import { transformExtent } from 'ol/proj';
import { containsXY } from 'ol/extent';

// Extents of each tile, per Maperitive dump-geo-bounds
const RAMBA_extent = transformExtent([-87.689608391558, 46.4727432356755, -87.5922970110678, 46.5259646094945], 'EPSG:4326', 'EPSG:3857');
const RAMBA_EPIC_extent = transformExtent([-87.6700190183706, 46.4727432356755, -87.5922970110678, 46.5063785453115], 'EPSG:4326', 'EPSG:3857');
const RAMBA_HD_extent = transformExtent([-87.6297194776245, 46.4984874260936, -87.6138562766484, 46.5035966369224], 'EPSG:4326', 'EPSG:3857');
const RAMBA_MALTON_extent = transformExtent([-87.6556351662006, 46.4903092720495, -87.6349630469947, 46.5088150825737], 'EPSG:4326', 'EPSG:3857');
const RAMBA_MEC_extent = transformExtent([-87.6440054057161, 46.4958666641897, -87.6342724903056, 46.5043638226569], 'EPSG:4326', 'EPSG:3857');
const RAMBA_QUAAL_extent = transformExtent([-87.6921131347103, 46.4857944374288, -87.633748278998, 46.5276740953805], 'EPSG:4326', 'EPSG:3857');
const RAMBA_SBT_extent = transformExtent([-87.6596313991497, 46.4866535695099, -87.6102244881526, 46.511912860167], 'EPSG:4326', 'EPSG:3857');
const RAMBA_SECTION16_extent = transformExtent([-87.6903179283854, 46.4742616882844, -87.631653922791, 46.5099890767745], 'EPSG:4326', 'EPSG:3857');
const RAMBA_PANTS_extent = transformExtent([-87.6504788840357, 46.4853475199722, -87.621671507138, 46.5013863803099], 'EPSG:4326', 'EPSG:3857');
const RAMBA_OTHER_extent = transformExtent([-87.6634527156019, 46.4753481470132, -87.6130431918585, 46.5099890767745], 'EPSG:4326', 'EPSG:3857');

// Overall extent, for maximum map scrolling
const MAP_extent = transformExtent([-87.7316239668677, 46.4527318050628, -87.5523749620375, 46.5400766282072], 'EPSG:4326', 'EPSG:3857');

// Put the MapTiler API Key Here
// const MapTiler_API_key = 'X';

const view = new View({
  center: getCenter(RAMBA_extent),
  extent: MAP_extent,
  minzoom: 10,
  maxzoom: 19,
});

const interactions = defaultInteractions({
  altShiftDragRotate: false,
  pinchRotate: false,
});

const controls = defaultControls({
  attributionOptions: {
    collapsible: false,
  }
});

const map = new Map({
  target: 'map',
  view: view,
  interactions: interactions,
  controls: controls,
  layers: [
    // MapTiler
    // new TileLayer({
    //   //   title: 'MapTiler',
    //   //   type: 'base',
    //   //   visible: false,
    //   source: new XYZ({
    //     url: 'https://api.maptiler.com/maps/topographique/{z}/{x}/{y}@2x.png?key=' + MapTiler_API_key,
    //     tilePixelRatio: 2,
    //     tileSize: 512,
    //     attributions: '<a href="https://maptiler.com/copyright">© MapTiler</a>',
    //   })
    // }),
    // OSM
    // new TileLayer({
    //   // title: 'OSM',
    //   // type: 'base',
    //   // visible: true,
    //   source: new OSM(),
    //   attributions: '© <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
    // }),
    // Local Path
    // new TileLayer({
    //   source: new XYZ({
    //     url: 'https://d2a0t4mb68t99u.cloudfront.net/basemap/{z}/{x}/{y}.png',
    //     opaque: true,
    //     tilePixelRatio: 2,
    //     attributions: 'Map: © <a href="https://nuxx.net">Steve Vigneau</a> / Data: © <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
    //   })
    // }),
    // Amazon Cloudfront
    new TileLayer({
      source: new XYZ({
        url: 'https://d2a0t4mb68t99u.cloudfront.net/basemap/{z}/{x}/{y}.png',
        opaque: true,
        tilePixelRatio: 2,
        attributions: 'Map: © <a href="https://nuxx.net">Steve Vigneau</a> / Data: © <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
      })
    }),
    
    new LayerGroup({
      title: 'RAMBA Trails',
      //fold: 'open',
      layers: [
        new TileLayer({
          title: 'All Trails',
          type: 'base',
          visible: true,
          extent: RAMBA_extent,
          source: new XYZ({
            url: 'https://d2a0t4mb68t99u.cloudfront.net/alltrails/{z}/{x}/{y}.png',
            opaque: false,
            tilePixelRatio: 2,
            // attributions: '© <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
          }),
        }),
        new TileLayer({
          title: 'Epic Loop',
          type: 'base',
          visible: false,
          extent: RAMBA_EPIC_extent,
          source: new XYZ({
            url: 'https://d2a0t4mb68t99u.cloudfront.net/epic/{z}/{x}/{y}.png',
            opaque: false,
            visible: false,
            tilePixelRatio: 2,
            // attributions: '© <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
          }),
        }),
        new TileLayer({
          title: 'Humpy Dumpty',
          type: 'base',
          visible: false,
          extent: RAMBA_HD_extent,
          source: new XYZ({
            url: 'https://d2a0t4mb68t99u.cloudfront.net/hd/{z}/{x}/{y}.png',
            opaque: false,
            tilePixelRatio: 2,
            // attributions: '© <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
          }),
        }),
        new TileLayer({
          title: 'Malton Loop',
          type: 'base',
          visible: false,
          extent: RAMBA_MALTON_extent,
          source: new XYZ({
            url: 'https://d2a0t4mb68t99u.cloudfront.net/malton/{z}/{x}/{y}.png',
            opaque: false,
            tilePixelRatio: 2,
            // attributions: '© <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
          }),
        }),
        new TileLayer({
          title: 'Malton-Epic Connector',
          type: 'base',
          visible: false,
          extent: RAMBA_MEC_extent,
          source: new XYZ({
            url: 'https://d2a0t4mb68t99u.cloudfront.net/mec/{z}/{x}/{y}.png',
            opaque: false,
            tilePixelRatio: 2,
            // attributions: '© <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
          }),
        }),
        new TileLayer({
          title: 'Quaal Loop',
          type: 'base',
          visible: false,
          extent: RAMBA_QUAAL_extent,
          source: new XYZ({
            url: 'https://d2a0t4mb68t99u.cloudfront.net/quaal/{z}/{x}/{y}.png',
            opaque: false,
            tilePixelRatio: 2,
            // attributions: '© <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
          }),
        }),
        new TileLayer({
          title: 'SBT (Snow Bike Trail)',
          type: 'base',
          visible: false,
          extent: RAMBA_SBT_extent,
          source: new XYZ({
            url: 'https://d2a0t4mb68t99u.cloudfront.net/sbt/{z}/{x}/{y}.png',
            opaque: false,
            tilePixelRatio: 2,
            // attributions: '© <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
          }),
        }),
        new TileLayer({
          title: 'Section 16 Loop',
          type: 'base',
          visible: false,
          extent: RAMBA_SECTION16_extent,
          source: new XYZ({
            url: 'https://d2a0t4mb68t99u.cloudfront.net/section16/{z}/{x}/{y}.png',
            opaque: false,
            tilePixelRatio: 2,
            // attributions: '© <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
          }),
        }),
        new TileLayer({
          title: 'Sissy Pants',
          type: 'base',
          visible: false,
          extent: RAMBA_PANTS_extent,
          source: new XYZ({
            url: 'https://d2a0t4mb68t99u.cloudfront.net/pants/{z}/{x}/{y}.png',
            opaque: false,
            tilePixelRatio: 2,
            // attributions: '© <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
          }),
        }),
        // new TileLayer({
        //   title: 'Other Trails',
        //   type: 'base',
        //   visible: false,
        //   extent: RAMBA_OTHER_extent,
        //   source: new XYZ({
        //     url: 'https://d2a0t4mb68t99u.cloudfront.net/other/{z}/{x}/{y}.png',
        //     opaque: false,
        //     tilePixelRatio: 2,
        //     // attributions: '© <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
        //   }),
        // }),
      ]
    }),
  ],
});

// Fit the RAMBA trails to the viewport.
map.getView().fit(RAMBA_extent);

// Create the vector source for the geolocation indocator.
const locationSource = new VectorSource({
});

// Put the vector source on a new layer
const locationLayer = new VectorLayer({
  map: map,
  source: locationSource
});

// Set up geolocation
var geolocation = new Geolocation

// Add the 'Locate me' control and when clicked either enable geolocation or re-center.
const locate = document.createElement('div');
locate.className = 'ol-control ol-unselectable locate';
locate.innerHTML = '<button title="Locate me">◎</button>';
locate.addEventListener('click', function () {
  if (geolocation.getTracking()) {
    const center = geolocation.getPosition();
    console.log(center[0] + ", " + center[1]);
    if (containsXY(MAP_extent, center[0], center[1])) {
      view.setCenter(center);
      // map.getView().fit(locationSource.getExtent(), {
      //   //maxZoom: 18,
      //   duration: 500
      // });
    };
  } else {
    enableGeolocation()
  };
});

// Add the other controls
map.addControl(new Control({ element: locate }));
map.addControl(new ScaleLine({ units: 'us' }));
map.addControl(new ZoomToExtent({
  extent: RAMBA_extent,
  label: "⏎",
  tipLabel: "Reset View",
}));
map.addControl(new LayerSwitcher({
  activationMode: 'click',
  tipLabel: 'Layers',
  collapseTipLabel: 'Close Layers',
  groupSelectStyle: 'none',
  startActive: true,
  reverse: false
}));

function enableGeolocation() {
  geolocation.setTrackingOptions({ enableHighAccuracy: true });
  geolocation.setProjection(view.getProjection());
  geolocation.setTracking(true);

  // Move to the current location, but only if within the extent of the map.
  geolocation.once('change:position', function () {
    var center = geolocation.getPosition();
    console.log(center[0] + ", " + center[1]);
    if (containsXY(MAP_extent, center[0], center[1])) {
      view.setCenter(center);
    }
  });

  // Set up the Accuracy feature and have it change as accuracy changes.
  var locationAccuracyFeature = new Feature();

  geolocation.on('change:accuracyGeometry', function () {
    locationAccuracyFeature.setGeometry(geolocation.getAccuracyGeometry());
  });

  // Set up the User Position feature and how it looks.
  var userPositionFeature = new Feature();

  userPositionFeature.setStyle(
    new Style({
      image: new CircleStyle({
        radius: 8,
        fill: new Fill({
          //
          color: '#FF3333',
        }),
        stroke: new Stroke({
          color: '#fff',
          width: 2,
        }),
      }),
    })
  );

  // Add Accuracy and Position to the Vector Layer.
  locationSource.addFeatures([locationAccuracyFeature, userPositionFeature]);

  geolocation.on('change:position', function () {
    const coordinates = geolocation.getPosition();
    userPositionFeature.setGeometry(coordinates ? new Point(coordinates) : null);
  });
};